﻿using Hotline.DataSharing.Extensions;
using Hotline.Share.Dtos.DataSharing.Enterprise;
using Hotline.Share.Dtos.DataSharing.Province;
using IdentityModel.Client;
using Microsoft.Extensions.DependencyInjection;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Net.Http.Json;
using System.Text.Json;
using XF.Domain.Cache;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.DataSharing.Enterprise
{
    public class EnterpriseClient : ISingletonDependency, ISelfDependency
    {

        private static readonly string KeyToken = "EnterpriseKeyToken";
        private readonly IServiceScopeFactory _scopeFactory;
        public EnterpriseClient(IServiceScopeFactory scopeFactory)
        {
            _scopeFactory = scopeFactory;
        }

        public async Task<TResponse?> RequestAsync<TRequest, TResponse>(TRequest request, CancellationToken cancellationToken)
            where TRequest : ISharingRequest, new()
        {
            using var scope = _scopeFactory.CreateScope();
            var provider = scope.ServiceProvider;
            var channelconfigManager = provider.GetRequiredService<IChannelConfigurationManager>();
            var httpInvoker = provider.GetRequiredService<IHttpInvoker>();
            var cacheToken = provider.GetRequiredService<ITypedCache<TokenTokenEnterpriseInfo>>();

            var token = cacheToken.GetOrSet(KeyToken,
                d => GetTokenAsync(cancellationToken).GetAwaiter().GetResult(),
                TimeSpan.FromMinutes(28));
            var configEnterprise = channelconfigManager.GetConfigurationEnterprise();

            return await httpInvoker.RequestAsync<TRequest, TResponse>(request,
                d => d.SetHttpClient(configEnterprise.AddressUrl, token),
                cancellationToken);
        }

        /// <summary>
        /// 获取Token
        /// </summary>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        /// <exception cref="UserFriendlyException"></exception>
        private async Task<TokenTokenEnterpriseInfo> GetTokenAsync(CancellationToken cancellationToken)
        {
            using var scope = _scopeFactory.CreateScope();
            var provider = scope.ServiceProvider;
            var channelconfigManager = provider.GetRequiredService<IChannelConfigurationManager>();
            var httpInvoker = provider.GetRequiredService<IHttpInvoker>();

            var configEnterprise = channelconfigManager.GetConfigurationEnterprise();
            var baseAddress = configEnterprise.AddressUrl;
            if (!baseAddress.EndsWith('/'))
                baseAddress += "/";

            string authorization = configEnterprise.ClientId + ":" + configEnterprise.ClientSecret;
            authorization = Encoder.Base64Code(authorization);
            authorization = authorization.Replace("@", "=");

            var request = new ClientCredentialsTokenRequest
            {
                Address = $"{baseAddress}blade-auth/oauth/getAccessToken",
                Headers =
                {
                    Accept =
                    {
                        MediaTypeWithQualityHeaderValue.Parse("application/json")
                    }
                },
                Method = HttpMethod.Post
            };
            request.Headers.Add("Authorization", "Basic " + authorization);
            request.Headers.Add("Tenant-Id", configEnterprise.TenantId);
            var tokenResponse = await httpInvoker.GetTokenAsync(request, cancellationToken);
            if (tokenResponse is null)
                throw new UserFriendlyException("获取token请求失败");
            if (tokenResponse.IsError)
                throw new UserFriendlyException("获取token请求失败");

            var tokenInfo = System.Text.Json.JsonSerializer.Deserialize<TokenEnterprise>(tokenResponse.Raw);
            if (tokenInfo is null || !tokenInfo.Success || tokenInfo?.TokenInfo is null)
                throw new UserFriendlyException("token解析失败");
            return tokenInfo.TokenInfo;
        }


    }
}
